"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const errors_1 = require("./errors");
function BabelPluginInclude(babel) {
    const { types } = babel;
    return {
        name: 'babel-plugin-include',
        visitor: {
            CallExpression(path, state) {
                if (path.node.callee.name === 'include') {
                    const root = state.opts.root || path_1.default.dirname(state.filename);
                    const encoding = state.opts.encoding || 'utf8';
                    const [filePathArg] = path.node.arguments;
                    if (filePathArg && types.isStringLiteral(filePathArg)) {
                        const includedFileName = filePathArg.value;
                        const pathToIncludedFile = path_1.default.resolve(root, includedFileName);
                        let fileContent;
                        try {
                            fileContent = fs_1.default.readFileSync(pathToIncludedFile, encoding);
                        }
                        catch (e) {
                            throw new errors_1.FileNotFoundError(pathToIncludedFile);
                        }
                        const { ast } = babel.transformSync(fileContent, {
                            ast: true,
                            filename: pathToIncludedFile,
                            configFile: false,
                            plugins: [[BabelPluginInclude, state.opts]],
                        });
                        path.replaceWithMultiple(ast.program.body);
                    }
                }
            },
        },
    };
}
exports.default = BabelPluginInclude;
